@@ -15,7 +15,7 @@ use List::MoreUtils        qw( any none );
 use Term::Choose           qw( choose );
 use Term::ReadLine::Simple qw();
 
-use App::YTDL::Helper qw( encode_list_for_lax_stdout );
+use App::YTDL::Helper qw( check_encode_stdout );
 
 
 
@@ -39,9 +39,11 @@ sub choose_videos {
     my $length_video_id   = 0;
     my $has_upload_date   = 0;
     my $has_view_count    = 0;
+    my $has_duration      = 0;
     for my $video_id ( keys %{$tmp->{$ex}} ) {
         $has_view_count++  if $tmp->{$ex}{$video_id}{view_count};
         $has_upload_date++ if $tmp->{$ex}{$video_id}{upload_date} ne '0000-00-00';
+        $has_duration++    if $tmp->{$ex}{$video_id}{duration}    ne '-:--:--';
         if ( length $video_id > $length_video_id ) {
             $length_video_id = length $video_id;
         }
@@ -68,6 +70,9 @@ sub choose_videos {
         if ( $sort_item eq 'view_count_raw' && ! $has_view_count ) {
             $sort_item = 'upload_date';
         }
+        if ( $sort_item eq 'duration' && ! $has_duration ) {
+            $sort_item = 'upload_date';
+        }
         if ( $sort_item eq 'upload_date' && ! $has_upload_date ) {
             $sort_item = 'title';
             $sort_order = 'Asc';
@@ -90,16 +95,13 @@ sub choose_videos {
                 $tmp->{$ex}{$video_id}{extractor} = 'youtube';
             }
             if ( $Encode::Locale::ENCODING_CONSOLE_OUT !~ /^UTF-/i ) {
-                ( $title ) = encode_list_for_lax_stdout( $opt, $title );
-                $title = decode( 'console_out', $title );
-            }
-            my $line = sprintf "%*s | %7s", $length_video_id, $video_id, $tmp->{$ex}{$video_id}{duration};
-            if ( $has_upload_date ) {
-                $line .= sprintf "  %10s", $tmp->{$ex}{$video_id}{upload_date};
-            }
-            if ( $has_view_count && ( $opt->{list_sort_item} eq 'view_count_raw' || $opt->{show_view_count} ) ) {
-                $line .= sprintf "  %*s", $length_view_count, $tmp->{$ex}{$video_id}{view_count};
+                $title = decode( 'console_out', check_encode_stdout( $opt, $title ) );
             }
+            $has_view_count = 0 if $opt->{list_sort_item} ne 'view_count_raw' && ! $opt->{show_view_count};
+            my $line = sprintf "%*s |", $length_video_id, $video_id;
+            $line .= sprintf " %7s", $tmp->{$ex}{$video_id}{duration}                        if $has_duration;
+            $line .= sprintf "  %10s", $tmp->{$ex}{$video_id}{upload_date}                   if $has_upload_date;
+            $line .= sprintf "  %*s", $length_view_count, $tmp->{$ex}{$video_id}{view_count} if $has_view_count;
             $line .= sprintf "  %s", $title;
             push @videos, $line;
             push @tmp_video_ids, $video_id;
