@@ -21,7 +21,7 @@ use Try::Tiny          qw( try catch );
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
 use App::YTDL::GetData qw( get_new_video_url );
-use App::YTDL::Helper  qw( sec_to_time encode_fs encode_stdout timestamp_to_upload_date HIDE_CURSOR SHOW_CURSOR );
+use App::YTDL::Helper  qw( sec_to_time encode_fs check_encode_stdout timestamp_to_upload_date HIDE_CURSOR SHOW_CURSOR );
 
 END { print SHOW_CURSOR }
 
@@ -35,12 +35,12 @@ sub download_youtube {
             for my $fmt ( sort keys %{$info->{$ex}{$video_id}{file_name}} ) {
                 my $file_name = $info->{$ex}{$video_id}{file_name}{$fmt};
                 try {
-                    unlink encode_fs( $file_name ) or die $!                      if -f encode_fs( $file_name ) && $opt->{overwrite};
+                    unlink encode_fs( $file_name ) or die check_encode_stdout( $opt, $file_name ) . " $!" if -f encode_fs( $file_name ) && $opt->{overwrite};
                     _download_video( $opt, $info, $ex, $video_id, $fmt, $nr );
                     timestamp_to_upload_date( $info, $ex, $video_id, $file_name ) if $opt->{modify_timestamp};
                 }
                 catch {
-                    say "$video_id - $_";
+                    say "$video_id - ", check_encode_stdout( $opt, $_ );
                 };
                 $nr++;
             }
@@ -63,9 +63,7 @@ sub _download_video {
     print HIDE_CURSOR;
     my $video_count = sprintf "%*s from %s", length $opt->{nr_videos}, $nr, $opt->{nr_videos};
     printf "  %s\n", '-' x length $video_count if $nr > 1;
-    binmode STDOUT, ':pop';
-    printf "  %s (%s)\n", encode_stdout( $file_basename ), $info->{$ex}{$video_id}{duration} // '?';
-    binmode STDOUT, ':encoding(console_out)';
+    printf "  %s (%s)\n", check_encode_stdout( $opt, $file_basename ), $info->{$ex}{$video_id}{duration} // '?';
     local $SIG{INT} = sub {
         print cldown, "\n";
         print SHOW_CURSOR;
@@ -88,7 +86,7 @@ sub _download_video {
         if ( ! $p->{size} ) {
             my $at = '';
             printf _p_fmt( $opt, "start" ), $video_count, $retries, $at;
-            open my $fh, '>:raw', encode_fs( $file_name ) or die $!;
+            open my $fh, '>:raw', encode_fs( $file_name ) or die check_encode_stdout( $opt, $file_name ) . " $!";
             _log_info( $opt, $info, $ex, $video_id, $fmt ) if $opt->{log_info};
             $res = $ua->get(
                 $video_url,
@@ -99,7 +97,7 @@ sub _download_video {
         elsif ( $p->{size} ) {
             my $at = sprintf "at %.2fM", $p->{size} / 1024 ** 2;
             printf _p_fmt( $opt, "start" ), $video_count, $retries, $at;
-            open my $fh, '>>:raw', encode_fs( $file_name ) or die $!;
+            open my $fh, '>>:raw', encode_fs( $file_name ) or die check_encode_stdout( $opt, $file_name ) . " $!";
             $res = $ua->get(
                 $video_url,
                 'Range'       => "bytes=$p->{size}-",
