@@ -6,7 +6,7 @@ use strict;
 use 5.010000;
 
 use Exporter qw( import );
-our @EXPORT_OK = qw( get_yt_list_info get_new_video_url get_download_info );
+our @EXPORT_OK = qw( get_yt_list_info get_vimeo_list_info get_new_video_url get_download_info );
 
 use JSON                qw( decode_json );
 use LWP::UserAgent      qw();
@@ -52,12 +52,56 @@ sub get_yt_list_info {
 }
 
 
+sub get_vimeo_list_info {
+    my ( $opt, $url ) = @_;
+    my $ua = LWP::UserAgent->new(
+        agent         => $opt->{useragent},
+        timeout       => $opt->{timeout},
+        show_progress => 1
+    );
+    eval { require Mojo::DOM } or die $@;
+    my $tmp = [];
+    my $next;
+    my $count = 0;
+
+    RETRY: while ( 1 ) {
+        my $ok = 1;
+        try {
+            $count++;
+            my $res = $ua->get( $url );
+            die $res->status_line, ': ', $url if ! $res->is_success;
+            my $decoded_content = $res->decoded_content;
+            my $dom = Mojo::DOM->new( $decoded_content );
+            my $uploader = $dom->at( 'a[class="user"]' )->text;
+            $dom->find( 'div[id="browse_content"] > ol > li[id][data-position]' )
+                ->each(
+                    sub {
+                        my $video_id = $_->attr( 'id' );
+                        my $title    = $_->at( 'a' )->attr( 'title' );
+                        my $datetime = $_->at( 'time' )->attr( 'datetime' );
+                        $video_id =~ s/clip_//;
+                        push @$tmp, { video_id => $video_id, title => $title, published => $datetime, uploader => $uploader };
+                    }
+                );
+            $next = $dom->at( 'a[rel="next"]' ) ? 1 : 0;
+        }
+        catch {
+            say "$count/$opt->{retries}  $_";
+            return if $count == $opt->{retries};
+            $ok = 0;
+            sleep $opt->{retries} * 3;
+        };
+        return $tmp, $next if $ok;
+    }
+}
+
+
 sub get_new_video_url {
     my ( $opt, $info, $ex, $video_id, $fmt ) = @_;
     my @cmd = ( $opt->{youtube_dl} );
-    push @cmd, '--user-agent', $opt->{useragent} if defined $opt->{useragent};
+    push @cmd, '--user-agent', $opt->{useragent}    if defined $opt->{useragent};
+    push @cmd, '--netrc'                            if $opt->{use_netrc};
     push @cmd, '--socket-timeout', $opt->{timeout};
-    #push @cmd, '-v';
     push @cmd, '--format', $fmt, '--get-url', '--', $video_id;
     my $video_url;
     try {
@@ -73,13 +117,13 @@ sub get_new_video_url {
 
 
 sub get_download_info {
-    my ( $opt, $video_id, $message ) = @_;
+    my ( $opt, $url, $message ) = @_;
     my @cmd = ( $opt->{youtube_dl} );
-    push @cmd, '--user-agent', $opt->{useragent} if defined $opt->{useragent};
+    push @cmd, '--user-agent', $opt->{useragent}    if defined $opt->{useragent};
+    push @cmd, '--netrc'                            if $opt->{use_netrc};
     push @cmd, '--socket-timeout', $opt->{timeout};
     push @cmd, '--youtube-skip-dash-manifest';
-    #push @cmd, '-v';
-    push @cmd, '--dump-json', '--', $video_id;
+    push @cmd, '--dump-json', '--', $url;
     my $json_all;
     my $count = 0;
 
@@ -93,12 +137,12 @@ sub get_download_info {
             print "\r", clline;
             print $message . "done.\n";
             print SHOW_CURSOR;
-            die $video_id . ' - no JSON!' if ! $json_all;
+            die $url . ' - no JSON!' if ! $json_all;
         }
         catch {
-            say "$count/$opt->{retries}  $video_id: $_"; #
+            say "$count/$opt->{retries}  $url: $_"; #
             if ( $count == $opt->{retries} ) {
-                push @{$opt->{error_get_download_infos}}, $video_id; # $ex
+                push @{$opt->{error_get_download_infos}}, $url; # $ex
                 return;
             }
             $ok = 0;
@@ -108,10 +152,7 @@ sub get_download_info {
         last RETRY if $ok;
     }
     $opt->{up}++;
-    my $tmp = {};
-    for my $json ( split /\n+/, $json_all ) {
-        json_to_hash( $opt, $tmp, $json );
-    }
+    my $tmp = json_to_hash( $opt, $json_all );
     return $tmp
 }
 
