@@ -6,7 +6,7 @@ use strict;
 use 5.010000;
 
 use Exporter qw( import );
-our @EXPORT_OK = qw( sec_to_time get_filename timestamp_to_upload_date encode_fs encode_list_for_lax_stdout encode_stdout
+our @EXPORT_OK = qw( sec_to_time get_filename timestamp_to_upload_date encode_fs check_encode_stdout
                     write_json read_json sanitize_for_path uni_capture uni_system HIDE_CURSOR SHOW_CURSOR );
 
 use Encode             qw( encode );
@@ -49,7 +49,7 @@ sub sanitize_for_path {
     $str =~ s/^\s+|\s+\z//g;
     $str =~ s/\s/_/g                         if $opt->{replace_spaces};
     #$str =~ s/^\.+//                         if $^O eq 'Unix';
-    $str =~ s/[\0\/]/-/g                     if $opt->{sanitize_filename} == 0;
+    $str =~ s/[\0\/<>&]/-/g                  if $opt->{sanitize_filename} == 0;
     $str =~ s/[\x{00}-\x{1F}"\/\\:*?<>|]/-/g if $opt->{sanitize_filename} == 1;
     # NTFS unsupported characters:  / \ : " * ? < > |       # 0x00-0x1F
     #$str =~ s/[^A-Za-z0-9_-]/-/g             if $opt->{sanitize_filename} == 2;
@@ -69,28 +69,43 @@ sub timestamp_to_upload_date {
 
 sub encode_fs {
     my ( $filename ) = @_;
-    return encode( 'locale_fs', NFC( $filename ), Encode::FB_HTMLCREF );
-}
-
-
-sub encode_list_for_lax_stdout {
-    my ( $opt, @strings ) = @_;
-    if ( $opt->{text_unidecode} ) {
-        eval {
-            require Text::Unidecode;
-            return Text::Unidecode::unidecode( @strings );
-        }
+    my $encoded_filename;
+    if ( eval { $encoded_filename = encode( 'locale_fs', NFC( $filename ), 1 ); 1 } ) {
+        return $encoded_filename;
     }
-    binmode STDOUT, ':pop';
-    my @encoded_lax = map { encode( 'console_out', NFC( $_ ), sub { '*' } ) } @strings;
-    binmode STDOUT, ':encoding(console_out)';
-    return @encoded_lax;
+    return $filename;
 }
 
 
-sub encode_stdout {
-    my ( $string ) = @_;
-    return encode( 'console_out', NFC( $string ), Encode::FB_HTMLCREF );
+sub check_encode_stdout {
+    my $opt = shift;
+    if ( wantarray ) {
+        return @_ if $Encode::Locale::ENCODING_CONSOLE_OUT =~ /^UTF-/i;
+        if ( $opt->{text_unidecode} ) {
+            my @unidecoded;
+            if ( eval { require Text::Unidecode; @unidecoded = Text::Unidecode::unidecode( @_ ); 1 } ) {
+                return @unidecoded;
+            }
+        }
+        binmode STDOUT, ':pop';
+        my @encoded_lax = map { encode( 'console_out', NFC( $_ ), sub { '*' } ) } @_;
+        binmode STDOUT, ':encoding(console_out)';
+        return @encoded_lax;
+    }
+    else {
+        my $string = shift;
+        return $string if $Encode::Locale::ENCODING_CONSOLE_OUT =~ /^UTF-/i;
+        if ( $opt->{text_unidecode} ) {
+            my $unidecoded;
+            if ( eval { require Text::Unidecode; $unidecoded = Text::Unidecode::unidecode( $string ); 1 } ) {
+                return $unidecoded;
+            }
+        }
+        binmode STDOUT, ':pop';
+        my $encoded_lax = encode( 'console_out', NFC( $string ), sub { '*' } );
+        binmode STDOUT, ':encoding(console_out)';
+        return $encoded_lax;
+    }
 }
 
 
@@ -151,18 +166,18 @@ sub sec_to_time {
 
 
 sub write_json {
-    my ( $file, $h_ref ) = @_;
+    my ( $opt, $file, $h_ref ) = @_;
     my $json = JSON->new->pretty->canonical->utf8->encode( $h_ref );
-    open my $fh, '>', encode_fs( $file ) or die $!;
+    open my $fh, '>', encode_fs( $file ) or die check_encode_stdout( $opt, $file ) . " $!";
     print $fh $json;
     close $fh;
 }
 
 
 sub read_json {
-    my ( $file ) = @_;
+    my ( $opt, $file ) = @_;
     return {} if ! -f encode_fs( $file );
-    open my $fh, '<', encode_fs( $file ) or die $!;
+    open my $fh, '<', encode_fs( $file ) or die check_encode_stdout( $opt, $file ) . " $!";
     my $json = do { local $/; <$fh> };
     close $fh;
     my $h_ref = JSON->new->utf8->decode( $json ) if $json;
