@@ -19,8 +19,8 @@ use Term::Choose::Util qw( term_size );
 
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
-use App::YTDL::Helper        qw( get_filename encode_list_for_lax_stdout encode_fs sanitize_for_path );
-use App::YTDL::History       qw( add_channel_to_history write_channel_history_to_file );
+use App::YTDL::Helper        qw( get_filename check_encode_stdout encode_fs sanitize_for_path );
+use App::YTDL::History       qw( add_uploader_to_history write_uploader_history_to_file );
 use App::YTDL::InfoPrint     qw( linefolded_print_info );
 use App::YTDL::Options       qw( set_options );
 use App::YTDL::SelectQuality qw( fmt_quality set_preferred_qualities );
@@ -63,7 +63,7 @@ sub get_download_infos {
             MENU: while ( 1 ) {
                 print "\n";
                 $opt->{up}++;
-                @$print_array = encode_list_for_lax_stdout( $opt, @$print_array ) if $Encode::Locale::ENCODING_CONSOLE_OUT !~ /^UTF-/i;
+                @$print_array = check_encode_stdout( $opt, @$print_array );
                 print for @$print_array;
                 $opt->{up} += @$print_array;
                 print "\n";
@@ -142,17 +142,17 @@ sub get_download_infos {
                 $video_dir = catdir $video_dir, $extractor_dir;
                 make_path encode_fs( $video_dir );
             }
-            if ( $opt->{channel_dir} == 2 || $opt->{channel_dir} == 1 && $info->{$ex}{$video_id}{from_list} ) {
+            if ( $opt->{uploader_dir} == 2 || $opt->{uploader_dir} == 1 && $info->{$ex}{$video_id}{from_list} ) {
                 if ( length $info->{$ex}{$video_id}{uploader} ) {
-                    my $channel_name = sanitize_for_path( $opt, $info->{$ex}{$video_id}{uploader} );
-                    $video_dir = catdir $video_dir, $channel_name;
+                    my $name_uploader = sanitize_for_path( $opt, $info->{$ex}{$video_id}{uploader} );
+                    $video_dir = catdir $video_dir, $name_uploader;
                     make_path encode_fs( $video_dir );
                 }
             }
             $info->{$ex}{$video_id}{video_dir} = $video_dir;
             printf "%*.*s : %s\n", $key_len, $key_len, 'video', $count;
             my $joined_fmts = sprintf " (%s)\n", join ' + ', @$fmt_list;
-            ( $joined_fmts ) = encode_list_for_lax_stdout( $opt, $joined_fmts ) if $Encode::Locale::ENCODING_CONSOLE_OUT !~ /^UTF-/i;
+            ( $joined_fmts ) = check_encode_stdout( $opt, $joined_fmts );
             $print_array->[0] =~ s/\n\z//;
             $print_array->[0] .= $joined_fmts;
             print for @$print_array;
@@ -167,11 +167,11 @@ sub get_download_infos {
                 $opt->{nr_videos}++;
 
             }
-            add_channel_to_history( $opt, $info, $ex, $video_id ) if $opt->{max_channels};
+            add_uploader_to_history( $opt, $info, $ex, $video_id ) if $opt->{max_size_history};
         }
     }
     print "\n";
-    write_channel_history_to_file( $opt ) if $opt->{max_channels};
+    write_uploader_history_to_file( $opt ) if $opt->{max_size_history};
     if ( ! $opt->{nr_videos} ) {
         print locate( 1, 1 ), cldown;
         say "No videos";
