@@ -15,7 +15,7 @@ use File::Spec::Functions qw( catfile catdir );
 use Term::Choose        qw( choose );
 use Try::Tiny           qw( try catch );
 
-use App::YTDL::Helper qw( get_filename timestamp_to_upload_date encode_fs uni_capture uni_system );
+use App::YTDL::Helper qw( get_filename timestamp_to_upload_date check_encode_stdout encode_fs uni_capture uni_system );
 
 
 
@@ -46,7 +46,7 @@ sub search_and_merge {
                     }
                 }
                 catch {
-                    say "$video_id - Merge: $_";
+                    say "$video_id - Merge: ", check_encode_stdout( $opt, $_ );
                     push @{$opt->{error_merge}}, $video_id;
                     $error = 1;
                 };
@@ -69,27 +69,27 @@ sub search_and_merge {
                 if ( -f $out && ! $opt->{merge_overwrite} ) {
                     my $overwrite_ok = choose(
                         [ '  Skip', '  Overwrite' ],
-                        { prompt => 'Already exists: ' . $file_name, layout => 3, index => 1 }
+                        { prompt => 'Already exists: ' . $file_name, layout => 3, index => 1 } #
                     );
                     next VIDEO_ID if ! $overwrite_ok;
                 }
                 try {
                     _merge_video_with_audio( $opt, $video, $audio, $out );
-                    say ' ' .  $out;
+                    say ' ' .  check_encode_stdout( $opt, $out );
                     if ( $opt->{merged_in_files} == 1 ) {
                         my $dir_orig_files = catdir $video_dir, $opt->{dir_stream_files};
-                        make_path $dir_orig_files;
-                        move $video, $dir_orig_files or die $!;
-                        move $audio, $dir_orig_files or die $!;
+                        make_path encode_fs( $dir_orig_files );
+                        move encode_fs( $video ), encode_fs( $dir_orig_files ) or die check_encode_stdout( $opt, $video ) . " $!";
+                        move encode_fs( $audio ), encode_fs( $dir_orig_files ) or die check_encode_stdout( $opt, $audio ) . " $!";
                     }
                     elsif ( $opt->{merged_in_files} == 2 ) {
-                        unlink $video or die $!;
-                        unlink $audio or die $!;
+                        unlink encode_fs( $video ) or die check_encode_stdout( $opt, $video ) . " $!";
+                        unlink encode_fs( $audio ) or die check_encode_stdout( $opt, $audio ) . " $!";
                     }
                     timestamp_to_upload_date( $info, $ex, $video_id, $out ) if $opt->{modify_timestamp};
                 }
                 catch {
-                    say "$video_id - Merge: $_";
+                    say "$video_id - Merge: ", check_encode_stdout( $opt, $_ );
                     push @{$opt->{error_merge}}, $video_id;
                 };
             }
