@@ -14,7 +14,7 @@ use FindBin               qw( $RealBin $RealScript );
 use Pod::Usage            qw( pod2usage );
 
 use Term::Choose           qw( choose );
-use Term::Choose::Util     qw( print_hash choose_a_dir choose_a_number insert_sep );
+use Term::Choose::Util     qw( print_hash choose_a_dir choose_a_number choose_multi insert_sep );
 use Term::ReadLine::Simple qw();
 use Text::LineFold         qw();
 use Try::Tiny              qw( try catch );
@@ -82,22 +82,22 @@ sub _show_info {
 sub _menus {
     my $menus = {
         main => [
-            [ 'show_help_text',  "  HELP"       ],
-            [ 'show_info',       "  INFO"       ],
-            [ 'group_directory', "- Directory"  ],
-            [ 'group_file',      "- File"       ],
-            [ 'group_quality',   "- Quality"    ],
-            [ 'group_download',  "- Download"   ],
-            [ 'group_merge',     "- Merge"      ],
-            [ 'group_history',   "- History"    ],
-            [ 'group_list_menu', "- Video List" ],
-            [ 'group_output',    "- Output" ],
-            [ 'list_extractors', "  Extractors" ],
+            [ 'show_help_text',   "  HELP"       ],
+            [ 'show_info',        "  INFO"       ],
+            [ 'group_directory',  "- Directory"  ],
+            [ 'group_file',       "- File"       ],
+            [ 'group_quality',    "- Quality"    ],
+            [ 'group_download',   "- Download"   ],
+            [ 'group_merge',      "- Merge"      ],
+            [ 'group_history',    "- History"    ],
+            [ 'group_list_menu',  "- Video List" ],
+            [ 'group_output',     "- Output"     ],
+            [ 'group_youtube_dl', "- Youtube-dl" ],
         ],
         group_directory => [
             [ 'video_dir',     "- Main video directory" ],
             [ 'extractor_dir', "- Extractor directory"  ],
-            [ 'channel_dir',   "- Channel directory"    ],
+            [ 'uploader_dir',  "- Uploader directory"   ],
         ],
         group_file => [
             [ 'max_len_f_name',    "- Max filename length" ],
@@ -106,9 +106,9 @@ sub _menus {
             [ 'modify_timestamp',  "- File timestamp"      ],
         ],
         group_quality => [
-            [ 'auto_quality',    "- Auto quality mode"            ],
-            [ 'print_pref_qual', "- Preferred qualities"          ],
-            [ 'pref_qual_slots', "- Slots 'preferred qualilties'" ],
+            [ 'auto_quality',     "- Auto quality mode"            ],
+            [ 'pref_qual_slots',  "- Slots 'preferred qualilties'" ],
+            [ '_print_pref_qual', "  Preferred qualities"          ],
         ],
         group_download => [
             [ 'useragent', "- UserAgent"        ],
@@ -124,12 +124,13 @@ sub _menus {
             [ 'merged_in_files', "- Input files"       ],
         ],
         group_history => [
-            [ 'log_info',             "- Logging"         ],
-            [ 'max_channels',         "- Channel history" ],
-            [ 'channel_by_timestamp', "- History sort"    ],
+            [ 'log_info',                  "- Logging"         ],
+            [ 'max_size_history',          "- Size history" ],
+            [ 'sort_history_by_timestamp', "- History sort"    ],
         ],
 
         group_list_menu => [
+            [ '_fast_listmenu',  "- Fast list-menu"  ],
             [ 'list_size_idx',   "- List size"       ],
             [ 'list_sort_item',  "- Sort order"      ],
             [ 'show_view_count', "- Show view count" ],
@@ -139,11 +140,26 @@ sub _menus {
             [ 'max_info_width', "- Max info width"        ],
             [ 'kb_sec_len',     "- Digits 'k/s'"          ],
         ],
+        group_youtube_dl => [
+            [ 'use_netrc',         "- Use '.netrc'" ],
+            [ '_avail_extractors', "  Extractors"        ],
+        ],
     };
     return $menus;
 }
 
 
+sub _sub_menus {
+    my ( $key ) = @_;
+    my $sub_menus = {
+        _fast_listmenu => [ [ 'fast_list_youtube', "- youtube", [ 'NO', 'YES' ] ],
+                            [ 'fast_list_vimeo',   "- vimeo",   [ 'NO', 'YES' ] ], ],
+    };
+    return $sub_menus->{$key} if $key;
+    return $sub_menus;
+}
+
+
 sub _group_prompt {
     my ( $group ) = @_;
     my $group_prompt ={
@@ -162,11 +178,15 @@ sub _group_prompt {
 
 sub set_options {
     my ( $opt ) = @_;
-    my $menus = _menus();
+    my $menus     = _menus();
+    my $sub_menus = _sub_menus();
     my @keys;
     for my $group ( keys %$menus ) {
         next if $group eq 'main';
-        push @keys, map { $_->[0] } @{$menus->{$group}};
+        push @keys, grep { ! /^_/ } map { $_->[0] } @{$menus->{$group}};
+    }
+    for my $key ( %$sub_menus ) {
+        push @keys, map { $_->[0] } @{$sub_menus->{$key}};
     }
     my $group = 'main';
     my $backup_old_idx = 0;
@@ -223,7 +243,7 @@ sub set_options {
             elsif ( $choice eq "show_info" ) {
                 _show_info( $opt );
             }
-            elsif ( $choice eq "list_extractors" ) {
+            elsif ( $choice eq "_avail_extractors" ) {
                 print HIDE_CURSOR;
                 say 'Close with ENTER';
                 my @capture;
@@ -251,11 +271,11 @@ sub set_options {
                 my $list = [ qw( no yes ) ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "channel_dir" ) {
-                my $prompt = 'Use channel directories';
+            elsif ( $choice eq "uploader_dir" ) {
+                my $prompt = 'Use uploader directories';
                 my $list = [
                     'no',
-                    'if from channel or playlist',
+                    'if from list-menu', #
                     'always',
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
@@ -273,7 +293,7 @@ sub set_options {
             elsif ( $choice eq "sanitize_filename" ) {
                 my $prompt = 'Sanitize filenames';
                 my $list = [
-                    'replace  /',
+                    'replace  / & < >',
                     'replace  / \ : " * ? < > |',
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
@@ -287,15 +307,15 @@ sub set_options {
                 my $prompt = 'Auto quality';
                 my $list = [
                     'manually',
-                    'keep_channel_playlist',
+                    'keep_uploader_playlist',
                     'keep_extractor',
                     'preferred',
                     'default',
                 ];
                 _opt_choose_from_list_value( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "print_pref_qual" ) {
-                my $pref = read_json( $opt->{preferred_file} );
+            elsif ( $choice eq "_print_pref_qual" ) {
+                my $pref = read_json( $opt, $opt->{preferred_file} );
                 if ( ! %$pref ) {
                     choose( [ 'Close with ENTER' ], { prompt => 'No preferred qualities set.' } );
                 }
@@ -376,21 +396,27 @@ sub set_options {
                 my $list = [ qw( no yes ) ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "max_channels" ) {
-                my $prompt = 'Channelhistory';
+            elsif ( $choice eq "max_size_history" ) {
+                my $prompt = 'Max size of history';
                 my $digits = 3;
                 _opt_number_range( $opt, $choice, $prompt, $digits );
             }
-            elsif ( $choice eq "channel_by_timestamp" ) {
-                my $prompt = 'Sort channel history';
+            elsif ( $choice eq "sort_history_by_timestamp" ) {
+                my $prompt = 'Sort history by';
                 my $list = [
                     'by name',
                     'by timestamp'
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
+            elsif ( $choice eq "_fast_listmenu" ) {
+                my $sub_menu = _sub_menus( $choice );
+                my $current = { map { $_->[0] => $opt->{$_->[0]} } @$sub_menu };
+                my $prompt = 'Enable fast list-menu for';
+                _opt_choose_multi( $opt, $sub_menu, $current, $prompt );
+            }
             elsif ( $choice eq "list_size_idx" ) {
-                my $prompt = 'Youtube videos per channel/playlist';
+                my $prompt = 'Videos in list-menu';
                 my $list = [
                     'all',
                     'latest 50',
@@ -427,6 +453,11 @@ sub set_options {
                 my $digits = 1;
                 _opt_number_range( $opt, $choice, $prompt, $digits );
             }
+            elsif ( $choice eq "use_netrc" ) {
+                my $prompt = 'Enable the option --netrc for youtube-dl';
+                my $list = [ qw( no yes ) ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
+            }
             else { die $choice }
         }
     }
@@ -434,6 +465,17 @@ sub set_options {
 }
 
 
+sub _opt_choose_multi {
+    my ( $opt, $sub_menu, $current, $prompt ) = @_;
+    my $changed = choose_multi( $sub_menu, $current, { prompt => $prompt, in_place => 0 } );
+    return if ! $changed;
+    for my $key ( keys %$changed ) {
+        $opt->{$key} = $changed->{$key};
+    }
+    $opt->{change}++;
+}
+
+
 sub _opt_choose_a_directory {
     my( $opt, $choice, $prompt ) = @_;
     my $new_dir = choose_a_dir( { dir => $opt->{$choice} } );
@@ -544,14 +586,14 @@ sub _write_config_file {
     for my $section ( sort @keys ) {
         $tmp->{$section} = $opt->{$section};
     }
-    write_json( $file, $tmp );
+    write_json( $opt, $file, $tmp );
     delete $opt->{change};
 }
 
 
 sub read_config_file {
     my ( $opt, $file ) = @_;
-    my $tmp = read_json( $file );
+    my $tmp = read_json( $opt, $file );
     my @keys = keys %$tmp;
     for my $section ( @keys ) {
         $opt->{$section} = $tmp->{$section};
